@@ -1,5 +1,8 @@
+use std::convert::TryFrom;
+
 use matches::assert_matches;
 use ruma_events::{AnyInitialStateEvent, InitialStateEvent};
+use ruma_identifiers::RoomNameBox;
 use serde_json::json;
 
 #[test]
@@ -11,6 +14,7 @@ fn deserialize_initial_state_event() {
         }))
         .unwrap(),
         AnyInitialStateEvent::RoomName(InitialStateEvent { content, state_key})
-        if content.name() == Some("foo") && state_key.is_empty()
+        if content.name == Some(RoomNameBox::try_from("foo").unwrap())
+            && state_key.is_empty()
     );
 }
